import numpy as np

from ... import defaults
from .data import Data

class Pie(Data):

    def __init__(self, values, labels, **kwargs):
        Data.__init__(self, **kwargs)
        defaults.kwargs(self, kwargs)
        self.set_data(values, labels)

    def set_data(self, values, labels):
        self.values = values
        self.labels = labels

defaults.load(Pie)



import os
from copy import deepcopy

from .. import defaults
from .figures import Figures
from .datatypes.line import Line as line
from .datatypes.lines import Lines as lines
from ..utils import readwrite

class Quick():

    def __init__(self, path_input, **kwargs):
        self.kwargs = kwargs
        defaults.kwargs(self, kwargs)
        self.process_file_blacklist()
        self.path_input = deepcopy(path_input)
        self.process_path_input()

    def process_file_blacklist(self):
        if self.file_blacklist is None:
            self.file_blacklist = []
        else:
            if isinstance(self.file_blacklist, str):
                self.file_blacklist = [self.file_blacklist]

    def process_path_input(self):
        self.set_paths()
        self.filter_paths()

    def set_paths(self):
        if isinstance(self.path_input, str):
            self.process_path_input_string()
        else:
            self.process_path_input_non_string()

    def process_path_input_string(self):
        if os.path.isdir(self.path_input):
            self.process_path_input_string_dir()
        else:
            self.process_path_input_string_non_dir()

    def process_path_input_string_dir(self):
        self.paths = [self.get_paths_from_path(os.path.join(self.path_input, path))
                      for path in os.listdir(self.path_input)]
        if isinstance(self.paths[0], str):
            self.paths = [self.paths]

    def get_paths_from_path(self, path):
        if os.path.isdir(path):
            return [os.path.join(path, child_path) for child_path in os.listdir(path)]
        else:
            return self.get_paths_from_paths_non_dir(path)

    def get_paths_from_paths_non_dir(self, path):
        if self.one_line_per_plot:
            return [os.path.join(self.path_input, path)]
        else:
            return os.path.join(self.path_input, path)

    def process_path_input_string_non_dir(self):
        if os.path.isfile(self.path_input):
            self.paths = [[self.path_input]]
        else:
            self.bad_path_exception(self.path_input)

    def bad_path_exception(self, path_input):
        message = (f"The following is not a valid path to a file or directory:\n"
                   f"{path_input}\n\n"
                   f"Original input:\n"
                   f"{self.path_input}")
        raise ValueError(message)

    def process_path_input_non_string(self):
        if hasattr(self.path_input, "_iter_"):
            self.process_path_input_iterable()
        else:
            self.non_iterable_non_string_exception(self.path_input)

    def process_path_input_iterable(self):
        if isinstance(self.path_input, dict):
            self.path_input = list(self.path_input.values())
        self.paths = [self.get_path(element) for element in self.path_input]
        self.ensure_two_dimensional_paths()

    def non_iterable_non_string_exception(self, path_input):
        message = (f"Input must be a string or iterable of strings\n"
                   f"Your input is of type {type(path_input)}")
        raise ValueError(message)

    def get_path(self, element):
        if isinstance(element, str):
            return self.get_path_string(element)
        else:
            return self.get_path_non_string(element)

    def get_path_string(self, element):
        if os.path.isdir(element):
            return [os.path.join(element, path) for path in os.path.listdir(element)]
        else:
            return self.get_path_string_non_dir(element)

    def get_path_string_non_dir(self, element):
        if self.one_line_per_plot:
            return [element]
        else:
            return element

    def get_path_non_string(self, element):
        if hasattr(element, "_iter_"):
            return self.get_path_non_string_iterable(element)
        else:
            return self.non_iterable_non_string_exception(element)

    def get_path_non_string_iterable(element):
        if isinstance(element, dict):
            element = list(element.values())
        return element

    def ensure_two_dimensional_paths(self):
        if not isinstance(self.paths[0], list):
            self.paths = [self.paths]

    def filter_paths(self):
        self.paths = [self.get_filtered_path_list(path_list)
                      for path_list in self.paths]
        self.paths = [path_list for path_list in self.paths
                      if len(path_list) != 0]

    def get_filtered_path_list(self, path_list):
        filtered_path_list = [path for path in path_list
                              if self.path_passes_blacklist(path)]
        return filtered_path_list

    def path_passes_blacklist(self, path):
        for blacklisted_string in self.file_blacklist:
            if blacklisted_string in path:
                return False
        return True
    
    def create_figures(self, **kwargs):
        lines_objects = [self.get_lines_obj(paths_list) for paths_list in self.paths]
        figures_obj = Figures(lines_objects, **kwargs)
        figures_obj.create_figures(**kwargs)

    def get_lines_obj(self, paths_list):
        line_objects = [self.get_line_obj(path) for path in paths_list]
        lines_obj = lines(line_objects, x_label=self.independent,
                          y_label=self.dependent, **self.kwargs)
        return lines_obj

    def get_line_obj(self, path):
        data_dict = readwrite.read_from_path(path)
        keys = list(data_dict.keys())
        x_values = self.get_x_values(data_dict, keys)
        y_values = self.get_y_values(data_dict, keys)
        line_obj = line(x_values, y_values, **self.kwargs)
        return line_obj

    def get_x_values(self, data_dict, keys):
        self.independent = keys[self.x]
        x_values = data_dict[self.independent][self.ignore_first:]
        return x_values

    def get_y_values(self, data_dict, keys):
        self.dependent = keys[self.y]
        y_values = data_dict[self.dependent][self.ignore_first:]
        return y_values
        
defaults.load(Quick)



import os

import matplotlib.pyplot as plt

def save_figure(figure_obj):
    path = get_figure_path(figure_obj)
    figures_obj = figure_obj.figures_obj
    save_fig(path, figures_obj)
    plt.close()

def get_figure_path(figure_obj):
    if figure_obj.path is not None:
        return figure_obj.path
    else:
        return get_default_figure_path(figure_obj)

def get_default_figure_path(figure_obj):
    file_name = get_file_name(figure_obj)
    file_name = f"{file_name}.{figure_obj.figures_obj.format}"
    path = os.path.join(figure_obj.figures_obj.base_path, file_name)
    return path

def get_file_name(figure_obj):
    if figure_obj.file_name is not None:
        return figure_obj.file_name
    else:
        return get_default_file_name(figure_obj)

def get_default_file_name(figure_obj):
    if len(figure_obj.figures_obj.data_object_groups) == 1:
        return get_base_file_name(figure_obj.figures_obj)
    else:
        return get_numbered_file_name(figure_obj)

def get_base_file_name(figure_obj):
    if figure_obj.suptitle is None:
        return "Figure"
    else:
        return str(figure_obj.suptitle)

def get_numbered_file_name(figure_obj):
    file_name = get_base_file_name(figure_obj.figure_objs)
    file_name = f"{file_name} {figure_obj.figure_index + 1}"
    return file_name

def save_fig(path, figures_obj):
    plt.savefig(path,
                dpi=figures_obj.dpi,
                format=figures_obj.format,
                metadata=figures_obj.metadata,
                bbox_inches=figures_obj.bbox_inches,
                pad_inches=figures_obj.pad_inches,
                facecolor=figures_obj.facecolor,
                edgecolor=figures_obj.edgecolor,
                backend=figures_obj.backend)



import os
import traceback
import inspect
import sys

class Docs():

    def __init__(self):
        self.extract_package()
        self.process_package_directory()
        self.add_package_docs()

    def extract_package(self):
        frame = inspect.stack()[3]
        module = inspect.getmodule(frame[0])
        self.package_path = os.path.split(module.__file__)[0]
        self.package_name = module.__name__
        self.package = sys.modules[self.package_name]

    def process_package_directory(self):
        for name, obj in inspect.getmembers(self.package):
            if not inspect.ismodule(obj):
                if hasattr(obj, "__module__"):
                    self.filter_non_module_objects(obj)

    def filter_non_module_objects(self, obj):
        object_module = inspect.getmodule(obj)
        common_path = self.get_common_path(object_module)
        if common_path == self.package_path:
            self.set_object_docs(obj)

    def get_common_path(self, object_module):
        object_module_path = object_module.__file__
        common_path = os.path.commonpath((self.package_path, object_module_path))
        return common_path

    def set_object_docs(self, obj):
        doc_path = self.get_doc_path(obj)
        doc_string = self.get_doc_string_from_path(doc_path)
        obj.__doc__ = doc_string

    def get_doc_path(self, obj):
        module = sys.modules[obj.__module__]
        folder_path = os.path.split(module.__file__)[0]
        name = os.path.splitext(obj.__name__)[0]
        doc_path = self.get_doc_path_from_path_data(folder_path, name)
        return doc_path

    def get_doc_path_from_path_data(self, folder_path, name):
        doc_file_name = f"{name}.txt"
        doc_path = os.path.join(folder_path, "Documentation", doc_file_name)
        return doc_path

    def get_doc_string_from_path(self, doc_path):
        if os.path.exists(doc_path):
            return self.get_doc_string_from_existing_path(doc_path)
        else:
            return self.get_doc_string_from_non_existing_path(doc_path)

    def get_doc_string_from_existing_path(self, doc_path):
        with open(doc_path, "r") as file:
            doc_string = "".join([line for line in file])
        return doc_string

    def get_doc_string_from_non_existing_path(self, doc_path):
        return ("No documentation exists for this object.\n"
                "It was expected to be found at the following location:\n"
                f"{doc_path}\n")

    def add_package_docs(self):
        doc_path = self.get_package_doc_path()
        doc_string = self.get_doc_string_from_path(doc_path)
        self.package.__doc__ = doc_string

    def get_package_doc_path(self):
        name = self.package.__name__.split(".")[-1]
        doc_path = self.get_doc_path_from_path_data(self.package_path, name)
        return doc_path

def docs():
    docs_obj = Docs()



from ..defaults import docs

# Functions to be accessed by the user
from .plotfunctions import create_figures
from .plotfunctions import create_animations
from .plotfunctions import quick

# Classes to be accessed by the user
from .datatypes.line import Line as line
from .datatypes.lines import Lines as lines
from .datatypes.bars import Bars as bars
from .datatypes.bar import Bar as bar
from .datatypes.pie import Pie as pie
from .datatypes.surface import Surface as surface
from .datatypes.colorplot import Colorplot as colorplot

# Classes that need initialisation
from .figure import Figure
from .plottypes.plotlines import PlotLines
from .plottypes.plotbars import PlotBars
from .plottypes.plotpie import PlotPie
from .plottypes.plotsurface import PlotSurface
from .plottypes.plotcolorplot import PlotColorplot

# Initialising classes
Figure.set_plot_classes()
PlotLines.set_function_dict()
PlotBars.set_function_dict()
PlotPie.set_function_dict()
PlotSurface.set_function_dict()
PlotColorplot.set_function_dict()

# Importing other classes that have documentation
# files so they can be detected by defaults.doc
from .animate import Animate
from .quick import Quick

docs()



def remove_none_values(dictionary):
    filtered_dictionary = {key: value for key, value in dictionary.items()
                           if value is not None}
    return filtered_dictionary

def transpose_list(my_list):
    transposed_list = list(map(list, zip(*my_list)))
    return transposed_list

def print_dict_aligned(dictionary):
    keys = list(dictionary.keys())
    max_length = max([len(str(key)) for key in keys])
    for key, value in dictionary.items():
        spaces = max_length - len(str(key))
        print(f"{key}: {spaces*' '}{value}")



from .readwrite import save_to_path
from .readwrite import read_from_path
from .readwrite import combine_files
from .readwrite import save_combined_files

from .filenames import get_file_name
from .filenames import read_file_name

from .printiterable import get_iterable_string
from .printiterable import print_iterable

from .paths import make_file
from .paths import make_folder
from .paths import load_json
from .paths import get_file_name_from_path
from .paths import remove_extension

from .iterables import transpose_list
from .iterables import remove_none_values
from .iterables import print_dict_aligned



import os
import json

def make_file(file_path):
    if not os.path.exists(file_path):
        folder_path = os.path.split(file_path)[0]
        make_folder(folder_path)
        make_empty_file(file_path)

def make_empty_file(file_path):
    with open(file_path, "w") as file:
        pass

def make_folder(folder_path, force=False):
    folder_path = get_folder_path_for_make_folder(folder_path, force)
    if not os.path.exists(folder_path):
        parent_folder_path = os.path.split(folder_path)[0]
        make_folder(parent_folder_path)
        os.mkdir(folder_path)

def get_folder_path_for_make_folder(folder_path, force):
    if force:
        return get_folder_path(folder_path)
    else:
        return folder_path

def get_folder_path(path):
    base_path, name = os.path.split(path)
    if "." in name:
        path = base_path
    return path

def make_folder_path(path):
    if os.path.isfile(path):
        path = os.path.split(path)[0]
    make_file(path)

def load_json(path, ignore_empty_or_none=True):
    if ignore_empty_or_none:
        if not os.path.exists(path) or os.stat(path).st_size == 0:
            return {}
    return do_load_json(path)

def do_load_json(path):
    with open(path, "r") as file:
        file_contents = json.load(file)
    return file_contents

def get_file_name_from_path(path):
    if os.path.exists(path):
        return remove_extension(os.path.split(path)[1])
    else:
        return remove_extension(path)

def remove_extension(path):
    no_extension_path = os.path.splitext(path)[0]
    return no_extension_path



import matplotlib.pyplot as plt
import numpy as np

from ... import defaults
from ..plottypes.plot import Plot

class PlotLines(Plot):

    @classmethod
    def set_function_dict(cls):
        cls.function_dict = {"plot": cls.plot_regular,
                             "semilogx": cls.plot_regular,
                             "semilogy": cls.plot_regular,
                             "loglog": cls.plot_regular,
                             "errorbar": cls.plot_errorbars,
                             "scatter": cls.plot_scatter}

    def __init__(self, figure_obj, ax, lines_obj, **kwargs):
        Plot.__init__(self, figure_obj, ax, lines_obj, **kwargs)
        self.lines_obj = lines_obj
        defaults.kwargs(self, kwargs)

    def plot_data(self):
        self.plot_lines()
        self.set_tick_labels()
        self.set_axis_limits()
        self.set_axis_bounds()

    def plot_lines(self):
        plot_type_function = self.get_plot_type_function()
        plot_function = getattr(self.ax, self.lines_obj.plot_type)
        for line_obj in self.lines_obj.line_objects:
            plot_type_function(self, plot_function, line_obj)

    def get_plot_type_function(self):
        plot_type = self.lines_obj.plot_type
        plot_type_function = self.function_dict[plot_type]
        return plot_type_function

    def plot_regular(self, plot_function, line_obj):
        plot_function(line_obj.x_values,
                      line_obj.y_values,
                      agg_filter=line_obj.agg_filter,
                      alpha=line_obj.alpha,
                      animated=self.lines_obj.animated,
                      antialiased=line_obj.antialiased,
                      clip_box=self.lines_obj.clip_box,
                      clip_on=self.lines_obj.clip_on,
                      clip_path=self.lines_obj.clip_path,
                      color=line_obj.color,
                      dash_capstyle=line_obj.dash_capstyle,
                      dash_joinstyle=line_obj.dash_joinstyle,
                      drawstyle=line_obj.drawstyle,
                      dashes=tuple(line_obj.dashes),
                      fillstyle=line_obj.fillstyle,
                      gapcolor=line_obj.gapcolor,
                      gid=line_obj.gid,
                      in_layout=line_obj.in_layout,
                      label=line_obj.label,
                      linestyle=line_obj.linestyle,
                      linewidth=line_obj.linewidth,
                      marker=line_obj.marker,
                      markeredgecolor=line_obj.markeredgecolor,
                      markeredgewidth=line_obj.markeredgewidth,
                      markerfacecolor=line_obj.markerfacecolor,
                      markerfacecoloralt=line_obj.markerfacecoloralt,
                      markersize=line_obj.markersize,
                      markevery=line_obj.markevery,
                      mouseover=line_obj.mouseover,
                      path_effects=line_obj.path_effects,
                      rasterized=line_obj.rasterized,
                      sketch_params=line_obj.sketch_params,
                      snap=line_obj.snap,
                      solid_capstyle=line_obj.solid_capstyle,
                      solid_joinstyle=line_obj.solid_joinstyle,
                      url=line_obj.url,
                      visible=line_obj.visible,
                      zorder=line_obj.zorder)

    def plot_errorbars(self, plot_function):
        plot_function(line_obj.x_values,
                      line_obj.y_values,
                      yerr=line_obj.yerr,
                      x_err=line_obj.x_err,
                      ecolor=line_obj.ecolor,
                      elinewidth=line_obj.elinewidth,
                      capsize=line_obj.capsize,
                      barsabove=line_obj.barsabove,
                      lolims=line_obj.lolims,
                      uplims=line_obj.uplims,
                      xlolims=line_obj.xlolims,
                      xuplims=line_obj.xuplims,
                      errorevery=line_obj.errorevery,
                      capthick=line_obj.capthick,
                      agg_filter=line_obj.agg_filter,
                      alpha=line_obj.alpha,
                      animated=self.lines_obj.animated,
                      antialiased=line_obj.antialiased,
                      clip_box=self.lines_obj.clip_box,
                      clip_on=self.lines_obj.clip_on,
                      clip_path=self.lines_obj.clip_path,
                      color=line_obj.color,
                      dash_capstyle=line_obj.dash_capstyle,
                      dash_joinstyle=line_obj.dash_joinstyle,
                      dashes=tuple(line_obj.dashes),
                      drawstyle=line_obj.drawstyle,
                      fillstyle=line_obj.fillstyle,
                      gapcolor=line_obj.gapcolor,
                      gid=line_obj.gid,
                      in_layout=line_obj.in_layout,
                      label=line_obj.label,
                      linestyle=line_obj.linestyle,
                      linewidth=line_obj.linewidth,
                      marker=line_obj.marker,
                      markeredgecolor=line_obj.markeredgecolor,
                      markeredgewidth=line_obj.markeredgewidth,
                      markerfacecolor=line_obj.markerfacecolor,
                      markerfacecoloralt=line_obj.markerfacecoloralt,
                      markersize=line_obj.markersize,
                      markevery=line_obj.markevery,
                      mouseover=line_obj.mouseover,
                      path_effects=line_obj.path_effects,
                      rasterized=line_obj.rasterized,
                      sketch_params=line_obj.sketch_params,
                      snap=line_obj.snap,
                      solid_capstyle=line_obj.solid_capstyle,
                      solid_joinstyle=line_obj.solid_joinstyle,
                      transform=line_obj.transform,
                      url=line_obj.url,
                      visible=line_obj.visible,
                      zorder=line_obj.zorder)

    def plot_scatter(self, plot_function):
        plot_function(line_obj.x_values,
                      line_obj.y_values,
                      s=line_obj.markersize,
                      c=line_obj.color,
                      marker=line_obj.marker,
                      cmap=line_obj.cmap,
                      norm=line_obj.norm,
                      vmin=line_obj.vmin,
                      vmax=line_obj.vmax,
                      alpha=line_obj.alpha,
                      linewidths=line_obj.linewidths,
                      edgecolors=line_obj.edgecolors,
                      plotnonfinite=line_obj.plotnonfinite)

    def add_axis_labels(self):
        self.add_x_label()
        self.add_y_label()

    def match_labels(self):
        labels = [line_obj.label for line_obj in self.data_obj.line_objects]
        for i, p in enumerate(self.ax.get_lines()):
            if p.get_label() in labels[:i]:
                idx = labels.index(p.get_label())
                p.set_c(self.ax.get_lines()[idx].get_c())
                p.set_label('_' + p.get_label())

defaults.load(PlotLines)



def inherit(child_obj, parent_obj, attributes):
    for attribute in attributes:
        if hasattr(parent_obj, attribute):
            parent_value = getattr(parent_obj, attribute)
            attempt_attribute_inheritance(child_obj, parent_value, attribute)

def attempt_attribute_inheritance(child_obj, parent_value, attribute):
    if hasattr(child_obj, attribute):
        child_has_attribute(child_obj, parent_value, attribute)
    else:
        setattr(child_obj, attribute, parent_value)

def child_has_attribute(child_obj, parent_value, attribute):
    if getattr(child_obj, attribute) is None:
        setattr(child_obj, attribute, parent_value)



import numpy as np

def get_iterable_string(iterable, indent=2):
    string = ""
    level = -1
    string = recurse(iterable, string, level, indent)
    return string

def recurse(iterable, string, level, indent):
    if if_iterable(iterable):
        return is_iterable(iterable, string, level, indent)
    else:
        return end_of_iterable(iterable, string, level, indent)

def if_iterable(iterable):
    has_iter = hasattr(iterable, "__iter__")
    is_not_string = not isinstance(iterable, str)
    return (has_iter and is_not_string)
        
def is_iterable(iterable, string, level, indent):
    if isinstance(iterable, dict):
        return dict_iterable(iterable, string, level+1, indent)
    else:
        return non_dict_iterable(iterable, string, level, indent)

def dict_iterable(iterable, string, level, indent):
    for key, value in iterable.items():
        string += f"{' '*indent*level}{key}:\n"
        string += add_dict_value(value, level, indent)
    return string

def add_dict_value(value, level, indent):
    if if_iterable(value):
        return recurse(value, "", level, indent)
    else:
        return recurse(value, "", level+1, indent)

def non_dict_iterable(iterable, string, level, indent):
    if np.any([if_iterable(item) for item in iterable]):
        return non_dict_iterable_iterate(iterable, string, level+1, indent)
    else:
        return end_of_iterable(iterable, string, level, indent)

def non_dict_iterable_iterate(iterable, string, level, indent):
    new_string = "".join([recurse(value, "", level, indent)
                          for value in iterable])
    string += new_string
    return string
    
def end_of_iterable(non_iterable, string, level, indent):
    string += f"{' '*indent*level}{non_iterable}\n"
    return string

def print_iterable(iterable, indent=2):
    string = get_iterable_string(iterable, indent=indent)
    print(string)
    return string



import os

import numpy as np

from .paths import make_folder

def save_to_path(path, data, separator=","):
    make_folder(path, force=True)
    with open(path, "w") as file:
        write_header_to_file(file, data, separator)
        write_columns_to_file(file, data, separator)

def write_header_to_file(file, data, separator):
    header_string = separator.join([str(key) for key in data.keys()])
    file.writelines(header_string + "\n")

def write_columns_to_file(file, data, separator):
    rows = zip(*list(data.values()))
    for row in rows:
        file.writelines(separator.join([str(value) for value in row]) + "\n")

def read_from_path(path, separater=",", skip_first_n=0,
                   data_type=float, enforce_type=True):
    with open(path, "r") as file:
        skip_first_lines(file, skip_first_n)
        keys = file.readline().strip("% \n").split(separater)
        values = get_data_from_file(file, separater, data_type, enforce_type)
        return dict(zip(keys, values))

def skip_first_lines(file, skip_first_n):
    for line_number in range(skip_first_n):
        file.readline()

def get_data_from_file(file, separater, data_type, enforce_type):
    rows = [line.strip().split(separater) for line in file]
    columns = [process_column(column, data_type, enforce_type)
               for column in zip(*rows)]
    return columns

def process_column(column, data_type, enforce_type):
    try:
        column = [data_type(value) for value in column]
    except:
        return process_column_error(column, data_type, enforce_type)
    return np.array(column)

def process_column_error(column, data_type, enforce_type):
    if enforce_type:
        enforce_type_error_message(column, data_type)
    else:
        return column

def enforce_type_error_message(column, data_type):
        raise ValueError(f"Could not convert data to {data_type}\n"
                         "Change the data type or see the enforce_type"
                         "kwarg to False\n"
                         f"First five entries: {column[:5]}")


def save_combined_files(folder_path, blacklist=None, name="Combined.txt"):
    results_path = os.path.join(folder_path, name)
    data = combine_files(folder_path, blacklist=blacklist)
    save_to_path(results_path, data)
    return data

def combine_files(folder_path, blacklist=None):
    blacklist = get_blacklist(blacklist)
    paths = get_combined_paths(folder_path, blacklist)
    data = get_data_from_paths(paths)
    return data

def get_blacklist(blacklist):
    if blacklist is None:
        blacklist = []
    return blacklist

def get_data_from_paths(paths):
    contents = [read_from_path(path) for path in paths]
    header = list(contents[0].keys())
    data = {key: np.concatenate([partial_contents[key]
                                 for partial_contents in contents],
                                axis=0)
            for key in header}
    return data

def get_combined_paths(folder_path, blacklist):
    paths = [os.path.join(folder_path, file_name)
             for file_name in os.listdir(folder_path)
             if file_name_not_blacklisted(file_name, blacklist)]
    return paths

def file_name_not_blacklisted(file_name, blacklist):
    for blacklisted_item in blacklist:
        if blacklist in file_name:
            return False
    return True



from .figures import Figures
from .animate import Animate
from .quick import Quick

def create_figures(data_objects, **kwargs):
    figures_obj = Figures(data_objects, **kwargs)
    figures_obj.create_figures(**kwargs)
    return figures_obj

def create_animations(data_objects, **kwargs):    
    figures_obj = Animate(data_objects, **kwargs)
    figures_obj.create_animations(**kwargs)
    return figures_obj

def quick(path_input, **kwargs):
    quick_obj = Quick(path_input, **kwargs)
    quick_obj.create_figures(**kwargs)



from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = "1.1.0"
DESCRIPTION = "A triple of tools used for plotting, handling key-words, and utilities"
LONG_DESCRIPTION = ("Defaults: manages settings for classes that can be controlled easily from an interface.\n"
                    "Plotting: a front end for matplotlib to easily create subplots.\n"
                    "Utils: a collection of generally useful functions")

# Setting up
setup(
    name="hgutilities",
    version=VERSION,
    author="Henry Ginn",
    author_email="<henryginn137@gmail.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    setup_requires=['setuptools_scm'],
    include_package_data=True,
    install_requires=["matplotlib", "numpy", "screeninfo", "pillow"],
    keywords=["python", "matplotlib", "plotting", "default", "keywords", "utils"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Natural Language :: English",
        "Operating System :: Unix",
        "Operating System :: Microsoft :: Windows",
    ]
)



import time
import os

from .. import defaults
from .paths import get_file_name_from_path

class FileName():

    def __init__(self, input_dict, **kwargs):
        defaults.kwargs(self, kwargs)
        self.set_input_dict(input_dict)
        self.set_file_name()

    def set_input_dict(self, input_dict):
        self.input_dict = (
            {self.ensure_pascal_case(key): self.convert_to_dict(value)
             for key, value in input_dict.items()})

    def ensure_pascal_case(self, key):
        words = [word for word_unsplit in key.split(" ")
                 for word in word_unsplit.split("_")]
        pascal_case = [f"{word[0].upper()}{word[1:].lower()}"
                       for word in words]
        key = "".join(pascal_case)
        return key

    def convert_to_dict(self, value):
        if not isinstance(value, dict):
            value = {"value": value}
        return value

    def set_file_name(self):
        component_names = [self.get_component_name(key, value_dict)
                           for key, value_dict in self.input_dict.items()]
        self.add_timestamp(component_names)
        self.file_name = f"{'__'.join(component_names)}.txt"

    def get_component_name(self, key, value_dict):
        if "unit" in value_dict:
            return self.component_name_with_units(key, value_dict)
        else:
            return self.component_name_without_units(key, value_dict)

    def component_name_with_units(self, key, value_dict):
        value = value_dict["value"]
        unit = value_dict["unit"]
        component_name = f"{key}_{value}_{unit}"
        return component_name

    def component_name_without_units(self, key, value_dict):
        value = value_dict["value"]
        component_name = f"{key}_{value}"
        return component_name

    def add_timestamp(self, component_names):
        if self.timestamp:
            timestamp = time.time()
            timestamp_string = f"T_{timestamp}"
            component_names.insert(0, timestamp_string)

defaults.load(FileName)

def get_file_name(input_dict, **kwargs):
    file_name_obj = FileName(input_dict, **kwargs)
    return file_name_obj.file_name

def read_file_name(file_name):
    name = get_file_name_from_path(file_name)
    data = dict(item.split("_")[:2]
                for item in name.split("__"))
    return {key: float(value) for key, value in data.items()}
